from math import log
from twibint import PyBigInt, PyBigUint, gen_random_pybiguint

A = 123456789101112131415161718
B = 987654321919293949596979899

def test_len():
	assert len(PyBigUint(A)) == int(log(A, 2**64)) + 1
	assert len(PyBigUint(B)) == int(log(B, 2**64)) + 1
	assert len(PyBigUint(2**128)) == int(log(2**128, 2**64)) + 1
def test_constructor():
	assert PyBigUint(A) == PyBigUint(str(A))
	assert PyBigUint(A) == A
	assert PyBigUint(float(A)) == int(float(A))
	assert PyBigUint(PyBigInt(A)) == PyBigUint(A)
	assert PyBigUint(PyBigUint(A)) == PyBigUint(A)

def test_add():
	assert PyBigUint(A) + PyBigUint(B) == PyBigUint(A + B)
	assert PyBigUint(A) + B == PyBigUint(A + B)
	assert A + PyBigUint(B) == PyBigUint(A + B)
	assert str(A) + PyBigUint(B) == PyBigUint(A + B)
	assert float(150) + PyBigUint(B) == 150 + B
def test_sub():
	assert PyBigUint(B) - PyBigUint(A) == PyBigUint(B - A)
	assert PyBigUint(B) - A == PyBigUint(B - A)
	assert B - PyBigUint(A) == PyBigUint(B - A)
def test_mul():
	assert PyBigUint(B) * PyBigUint(A) == PyBigUint(B * A)
	assert B * PyBigUint(A) == PyBigUint(B * A)
	assert PyBigUint(B) * A == PyBigUint(B * A)
def test_div():
	assert PyBigUint(B) // PyBigUint(A) == PyBigUint(B // A)
	assert B // PyBigUint(A) == PyBigUint(B // A)
	assert PyBigUint(B) // A == PyBigUint(B // A)
def test_mod():
	assert PyBigUint(B) % PyBigUint(A) == PyBigUint(B % A)
	assert B % PyBigUint(A) == PyBigUint(B % A)
	assert PyBigUint(B) % A == PyBigUint(B % A)
def test_divmod():
	a, b = divmod(B, A)
	assert divmod(PyBigUint(B), PyBigUint(A)) == (PyBigUint(a), PyBigUint(b))
	assert divmod(B, PyBigUint(A)) == (PyBigUint(a), PyBigUint(b))
	assert divmod(PyBigUint(B), A) == (PyBigUint(a), PyBigUint(b))
def test_truediv():
	assert PyBigUint(A) / PyBigUint(B) == A / B
	assert PyBigUint(B) / PyBigUint(A) == B / A
	assert B / PyBigUint(A) == B / A
	assert PyBigUint(B) / A == B / A
def test_pow():
	assert PyBigUint(A) ** 2 == PyBigUint(A ** 2)
	assert PyBigUint(B) ** 2 == PyBigUint(B ** 2)

def test_shr():
	assert (PyBigUint(A) >> 10 == PyBigUint(A >> 10))
def test_shl():
	assert (PyBigUint(A) << 10 == PyBigUint(A << 10))

def test_and():
	assert (PyBigUint(A) & PyBigUint(B)) == PyBigUint(A & B)
	assert (A & PyBigUint(B)) == PyBigUint(A & B)
	assert (PyBigUint(A) & B) == PyBigUint(A & B)
	a = PyBigUint(A)
	a &= B
	assert a == A & B
def test_xor():
	assert (PyBigUint(A) ^ PyBigUint(B)) == PyBigUint(A ^ B)
	assert (A ^ PyBigUint(B)) == PyBigUint(A ^ B)
	assert (PyBigUint(A) ^ B) == PyBigUint(A ^ B)
	a = PyBigUint(A)
	a ^= B
	assert a == A ^ B
def test_or():
	assert (PyBigUint(A) | PyBigUint(B)) == PyBigUint(A | B)
	assert (A | PyBigUint(B)) == PyBigUint(A | B)
	assert (PyBigUint(A) | B) == PyBigUint(A | B)
	a = PyBigUint(A)
	a |= B
	assert a == A | B

def test_float():
	assert float(PyBigUint(A)) == float(A)
	assert float(PyBigUint(B)) == float(B)
def test_int():
	assert int(PyBigUint(A)) == A
	assert int(PyBigUint(B)) == B
def test_from_float():
	assert PyBigUint(float(A)) == PyBigUint(int(float(A)))

def test_eq():
	assert PyBigUint(A) + PyBigUint(B) == PyBigUint(A + B)
	assert PyBigUint(A) + PyBigUint(B) == A + B
def test_le():
	assert PyBigUint(A) + PyBigUint(B) <= PyBigUint(A + B)
	assert PyBigUint(A) + PyBigUint(B) <= A + B
def test_ge():
	assert PyBigUint(A) + PyBigUint(B) >= PyBigUint(A + B)
	assert PyBigUint(A) + PyBigUint(B) >= A + B
def test_lt():
	assert PyBigUint(A) < PyBigUint(A + B)
	assert A < PyBigUint(A + B)
	assert PyBigUint(A) < A + B
def test_gt():
	assert PyBigUint(A + B) > PyBigUint(A)
	assert PyBigUint(A + B) > A

def test_bool():
	assert bool(PyBigUint(5)) == bool(1)
	assert bool(PyBigUint(1)) == bool(1)
	assert bool(PyBigUint(0)) == bool(0)

def test_rand():
	C = gen_random_pybiguint(100)
	assert True


import twibint as bi 

def bench_product(A, B, nb_samples):
	for i in range(nb_samples):
		print(str(i) + "/" + str(nb_samples))
		C = A * B
	print()

SIZE = 50000
NB_SAMPLES = 50

A_bi = bi.gen_random_biguint(SIZE)
B_bi = bi.gen_random_biguint(SIZE)
A_int = int(A_bi)
B_int = int(B_bi)

# bench_product(A_bi, B_bi, NB_SAMPLES)
bench_product(A_int, B_int, NB_SAMPLES)


import os
import json
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt

walker = os.walk("./target/criterion")
folders = next(walker)[1]

data = {
	name: {
		"new": {"x": [], "low_y": [], "up_y": []},
		"base": {"x": [], "low_y": [], "up_y": []},
	} for name in ["add", "mul", "sub"]
}

for f in folders:
	if f  == "report":
		continue

	try :
		with open("./target/criterion/" + f + "/"+"new"+"/estimates.json", "r") as file:
			new_data = json.load(file)
		lower_bound = new_data["mean"]["confidence_interval"]["lower_bound"]
		upper_bound = new_data["mean"]["confidence_interval"]["upper_bound"]

		name, x = f.split(" ")
		if name in data.keys():
			data[name]["new"]["x"].append(int(x));
			data[name]["new"]["low_y"].append(lower_bound);
			data[name]["new"]["up_y"].append(upper_bound);
	except FileNotFoundError:
		pass


for name, points in data.items():
	fig = plt.figure()
	fig.suptitle(name)

	indices = np.argsort(points["new"]["x"])
	X = np.array(points["new"]["x"])[indices]
	Y1 = np.array(points["new"]["low_y"])[indices]
	Y2 = np.array(points["new"]["up_y"])[indices]

	m,b = np.polyfit(
		np.log(X), 
		np.log(Y1), 
		1)
	plt.plot(X, Y1, "x", label="lower bound")
	plt.plot(X, Y2, "x", label="upper bound")
	plt.plot(X, np.exp(b)*(X**m), color="red", label="linear fit "+str(m))

	plt.xlabel("time in ns")
	plt.ylabel("number of digits (in base 2³²)")
	plt.yscale("log")
	plt.xscale("log")
	plt.legend()

plt.show()


from math import log
from twibint import PyBigInt, PyBigUint

A = 123456789101112131415161718
B = -987654321919293949596979899

def test_len():
	assert len(PyBigInt(A)) == int(log(A, 2**64)) + 1
	assert len(PyBigInt(B)) == int(log(-B, 2**64)) + 1
	assert len(PyBigInt(2**128)) == int(log(2**128, 2**64)) + 1
def test_constructor():
	assert PyBigInt(A) == PyBigInt(str(A))
	assert PyBigInt(A) == A
	assert PyBigInt(float(A)) == int(float(A))
	assert PyBigInt(PyBigInt(A)) == PyBigInt(A)
	assert PyBigInt(PyBigUint(A)) == PyBigInt(A)

def test_add():
	assert PyBigInt(A) + PyBigInt(B) == PyBigInt(A + B)
	assert PyBigInt(A) + B == PyBigInt(A + B)
	assert A + PyBigInt(B) == PyBigInt(A + B)
	assert float(150) + PyBigInt(B) == 150 + B
def test_sub():
	assert PyBigInt(B) - PyBigInt(A) == PyBigInt(B - A)
	assert B - PyBigInt(A) == PyBigInt(B - A)
	assert PyBigInt(B) - A == PyBigInt(B - A)
def test_mul():
	assert PyBigInt(B) * PyBigInt(A) == PyBigInt(B * A)
	assert B * PyBigInt(A) == PyBigInt(B * A)
	assert PyBigInt(B) * A == PyBigInt(B * A)
def test_div():
	assert PyBigInt(B) // PyBigInt(A) == PyBigInt(B // A)
	assert PyBigInt(B) // A == PyBigInt(B // A)
	assert B // PyBigInt(A) == PyBigInt(B // A)
def test_mod():
	assert PyBigInt(B) % PyBigInt(A) == PyBigInt(B % A)
	assert B % PyBigInt(A) == PyBigInt(B % A)
	assert PyBigInt(B) % A == PyBigInt(B % A)
def test_divmod():
	a, b = divmod(B, A)
	assert divmod(PyBigInt(B), PyBigInt(A)) == (PyBigInt(a), PyBigInt(b))
	assert divmod(B, PyBigInt(A)) == (PyBigInt(a), PyBigInt(b))
	assert divmod(PyBigInt(B), A) == (PyBigInt(a), PyBigInt(b))
def test_truediv():
	assert PyBigInt(A) / PyBigInt(B) == A / B
	assert PyBigInt(B) / PyBigInt(A) == B / A
	assert PyBigInt(B) / A == B / A
	assert B / PyBigInt(A) == B / A
def test_pow():
	assert PyBigInt(A) ** 2 == PyBigInt(A ** 2)
	assert PyBigInt(B) ** 2 == PyBigInt(B ** 2)

def test_shr():
	assert (PyBigInt(A) >> 10 == PyBigInt(A >> 10))
	assert (PyBigInt(B) >> 10 == PyBigInt(B >> 10))
def test_shl():
	assert (PyBigInt(A) << 10 == PyBigInt(A << 10))
	assert (PyBigInt(B) << 10 == PyBigInt(B << 10))

def test_xor():
	assert (PyBigInt(A) ^ PyBigInt(B)) == PyBigInt(A ^ B)
	assert (PyBigInt(-A) ^ PyBigInt(B)) == PyBigInt((-A) ^ B)
	assert (PyBigInt(A) ^ PyBigInt(-B)) == PyBigInt(A ^ (-B))
	assert (PyBigInt(-A) ^ PyBigInt(-B)) == PyBigInt((-A) ^ (-B))
def test_or():
	assert (PyBigInt(A) | PyBigInt(B)) == PyBigInt(A | B)
	assert (PyBigInt(-A) | PyBigInt(B)) == PyBigInt((-A) | B)
	assert (PyBigInt(A) | PyBigInt(-B)) == PyBigInt(A | (-B))
	assert (PyBigInt(-A) | PyBigInt(-B)) == PyBigInt((-A) | (-B))
def test_and():
	assert (PyBigInt(A) & PyBigInt(B)) == PyBigInt(A & B)
	assert (PyBigInt(-A) & PyBigInt(B)) == PyBigInt((-A) & B)
	assert (PyBigInt(A) & PyBigInt(-B)) == PyBigInt(A & (-B))
	assert (PyBigInt(-A) & PyBigInt(-B)) == PyBigInt((-A) & (-B))
def test_not():
	assert ~PyBigInt(A) == ~A
	assert ~PyBigInt(B) == ~B

def test_float():
	assert float(PyBigInt(A)) == float(A)
	assert float(PyBigInt(B)) == float(B)
def test_from_float():
	assert PyBigInt(float(A)) == PyBigInt(int(float(A)))
	assert PyBigInt(float(B)) == PyBigInt(int(float(B)))
def test_int():
	assert int(PyBigInt(A)) == A
	assert int(PyBigInt(B)) == B

def test_eq():
	assert PyBigInt(A) + PyBigInt(B) == PyBigInt(A + B)
	assert PyBigInt(A) + PyBigInt(B) == A + B
def test_le():
	assert PyBigInt(A) + PyBigInt(B) <= PyBigInt(A + B)
	assert PyBigInt(A) + PyBigInt(B) <= A + B
def test_ge():
	assert PyBigInt(A) + PyBigInt(B) >= PyBigInt(A + B)
	assert PyBigInt(A) + PyBigInt(B) >= A + B
def test_lt():
	assert abs(PyBigInt(A)) < PyBigInt(abs(A) + abs(B))
	assert abs(A) < PyBigInt(abs(A) + abs(B))
	assert abs(PyBigInt(A)) < abs(A) + abs(B)
def test_gt():
	assert PyBigInt(abs(A) + abs(B)) > abs(PyBigInt(A))
	assert PyBigInt(abs(A) + abs(B)) > abs(A)

def test_bool():
	assert bool(PyBigInt(1)) == bool(1)
	assert bool(PyBigInt(0)) == bool(0)


import sys
import subprocess

import pathlib
directory = str(pathlib.Path(__file__).parent.resolve())

def pytest_configure(config):
    subprocess.run([
        sys.executable, 
        "-m", 
        "maturin", 
        "develop", 
        "-r"], 
        cwd = directory)